/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.result;

import org.HdrHistogram.DoubleHistogram;
import org.neo4j.gds.core.utils.partition.Partition;

import java.util.function.LongToDoubleFunction;
import java.util.function.Supplier;

class CentralityRecordTask implements Runnable {

    private final DoubleHistogram histogram;
    private final Partition partition;
    private final LongToDoubleFunction centralityFunction;

    CentralityRecordTask(
        Partition partition,
        LongToDoubleFunction centralityFunction,
        Supplier<DoubleHistogram> histogramSupplier
    ) {
        this.partition = partition;
        this.centralityFunction = centralityFunction;
        this.histogram = histogramSupplier.get();
    }

    @Override
    public void run() {
        partition.consume(id -> {
            histogram.recordValue(centralityFunction.applyAsDouble(id));
        });
    }

    DoubleHistogram histogram() {return  histogram;}
}
