/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.concurrency;

import static org.neo4j.gds.utils.StringFormatting.formatWithLocale;

public class OpenGdsConcurrencyValidator implements ConcurrencyValidator {

    @Override
    public void validate(int requestedConcurrency, String configKey, int concurrencyLimitation) throws IllegalArgumentException {
        if (requestedConcurrency > concurrencyLimitation) {
            throw new IllegalArgumentException(formatWithLocale(
                "Community users cannot exceed %1$s=%2$d (you configured %1$s=%3$d), see https://neo4j.com/docs/graph-data-science/",
                configKey,
                concurrencyLimitation,
                requestedConcurrency
            ));
        }
    }

}
