/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.doc.syntax;

import org.immutables.value.Value;
import org.neo4j.configuration.GraphDatabaseSettings;
import org.neo4j.gds.annotation.ValueClass;

/**
 * A doc query has query text and an optional operator
 */
@ValueClass
public interface DocQuery {
    String DEFAULT_OPERATOR = "";

    String query();

    @Value.Derived
    default boolean runAsOperator() {
        return !operator().equals("");
    }

    @Value.Default
    default String operator() {
        return DEFAULT_OPERATOR;
    }

    @Value.Default
    default String database() { return GraphDatabaseSettings.DEFAULT_DATABASE_NAME;}

    static ImmutableDocQuery.Builder builder() {
        return ImmutableDocQuery.builder();
    }
}
