/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.metrics;

import org.neo4j.gds.metrics.algorithms.AlgorithmMetricsService;
import org.neo4j.gds.metrics.procedures.DeprecatedProceduresMetricService;
import org.neo4j.gds.metrics.projections.ProjectionMetricsService;

/**
 * This is a <a href="https://refactoring.com/catalog/introduceParameterObject.html">parameter object</a> for all things metrics.
 */
public class Metrics {
    public static final Metrics DISABLED = new Metrics(
        AlgorithmMetricsService.DISABLED,
        ProjectionMetricsService.DISABLED,
        DeprecatedProceduresMetricService.DISABLED
    );

    private final AlgorithmMetricsService algorithmMetricsService;
    private final ProjectionMetricsService projectionMetricsService;
    private final DeprecatedProceduresMetricService deprecatedProceduresMetricService;

    public Metrics(
        AlgorithmMetricsService algorithmMetricsService,
        ProjectionMetricsService projectionMetricsService,
        DeprecatedProceduresMetricService deprecatedProceduresMetricService
    ) {
        this.algorithmMetricsService = algorithmMetricsService;
        this.projectionMetricsService = projectionMetricsService;
        this.deprecatedProceduresMetricService = deprecatedProceduresMetricService;
    }

    public AlgorithmMetricsService algorithmMetrics() {
        return algorithmMetricsService;
    }

    public ProjectionMetricsService projectionMetrics() {
        return projectionMetricsService;
    }

    public DeprecatedProceduresMetricService deprecatedProcedures() {
        return deprecatedProceduresMetricService;
    }
}
