/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.metrics.projections;

import org.neo4j.gds.metrics.ExecutionMetric;
import org.neo4j.gds.metrics.ExecutionMetricRegistrar;

public class ProjectionMetricsService {
    public static final ProjectionMetricsService DISABLED = new ProjectionMetricsService(ExecutionMetricRegistrar.DISABLED);

    private final ExecutionMetricRegistrar metricRegistrar;

    public ProjectionMetricsService(ExecutionMetricRegistrar metricRegistrar) {
        this.metricRegistrar = metricRegistrar;
    }

    public ExecutionMetric createNative() {
        return metricRegistrar.create("native");
    }

    public ExecutionMetric createCypher() {
        return metricRegistrar.create("cypher");
    }

    public ExecutionMetric createCypherV2() {
        return metricRegistrar.create("cypherV2");
    }

    public ExecutionMetric createSubGraph() {
        return metricRegistrar.create("subGraph");
    }

    public ExecutionMetric createRandomWakSampling(String type) {
        return metricRegistrar.create(type);
    }
}
