/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.procedures.algorithms.similarity;

import org.apache.commons.lang3.tuple.Pair;
import org.neo4j.gds.api.Graph;
import org.neo4j.gds.applications.algorithms.machinery.AlgorithmProcessingTimings;
import org.neo4j.gds.applications.algorithms.machinery.ResultBuilder;
import org.neo4j.gds.applications.algorithms.metadata.RelationshipsWritten;
import org.neo4j.gds.similarity.filteredknn.FilteredKnnResult;
import org.neo4j.gds.similarity.filteredknn.FilteredKnnWriteConfig;

import java.util.Map;
import java.util.Optional;
import java.util.stream.Stream;

class FilteredKnnResultBuilderForWriteMode implements ResultBuilder<FilteredKnnWriteConfig, FilteredKnnResult, Stream<KnnWriteResult>, Pair<RelationshipsWritten, Map<String, Object>>> {
    @Override
    public Stream<KnnWriteResult> build(
        Graph graph,
        FilteredKnnWriteConfig configuration,
        Optional<FilteredKnnResult> result,
        AlgorithmProcessingTimings timings,
        Optional<Pair<RelationshipsWritten, Map<String, Object>>> metadata
    ) {
        var configurationMap = configuration.toMap();

        if (result.isEmpty()) return Stream.of(
            KnnWriteResult.emptyFrom(
                timings,
                configurationMap
            )
        );

        var knnWriteResult = KnnWriteResult.from(
            timings,
            metadata.orElseThrow().getLeft(),
            metadata.orElseThrow().getRight(),
            result.get().nodesCompared(),
            result.get().didConverge(),
            result.get().ranIterations(),
            result.get().nodePairsConsidered(),
            configurationMap
        );

        return Stream.of(knnWriteResult);
    }
}
