/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.labelpropagation;

import org.neo4j.gds.annotation.Configuration;
import org.neo4j.gds.config.AlgoBaseConfig;
import org.neo4j.gds.config.ConsecutiveIdsConfig;
import org.neo4j.gds.config.IterationsConfig;
import org.neo4j.gds.config.NodeWeightConfig;
import org.neo4j.gds.config.RelationshipWeightConfig;
import org.neo4j.gds.config.SeedConfig;

public interface LabelPropagationBaseConfig extends
    AlgoBaseConfig,
    SeedConfig,
    ConsecutiveIdsConfig,
    RelationshipWeightConfig,
    NodeWeightConfig,
    IterationsConfig {

    @Override
    @Configuration.IntegerRange(min = 1)
    default int maxIterations() {
        return 10;
    }

    @Configuration.Ignore
    default LabelPropagationParameters toParameters() {
        return new LabelPropagationParameters(concurrency(), maxIterations(), nodeWeightProperty(), seedProperty());
    }
}
